# Generated by Django 4.0.7 on 2022-11-12 04:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0003_alter_general_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="settings",
            name="interest_rate",
            field=models.FloatField(
                default=5.0,
                verbose_name="Monthly interest rate (%) if taxes have not been paid",
            ),
        ),
        migrations.AlterField(
            model_name="settings",
            name="phrase",
            field=models.CharField(
                blank=True,
                default="",
                max_length=10,
                verbose_name="Keyword (case insensitive) that must be present in the donation reason to be counted. Leave blank/empty to count all donations regardless of reason.",
            ),
        ),
    ]



# Generated by Django 4.0.7 on 2023-10-12 22:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0010_stats_admin_get_all_activity_json"),
    ]

    operations = [
        migrations.AddField(
            model_name="stats",
            name="curmonth_leadergraph",
            field=models.JSONField(default=None, null=True),
        ),
    ]



# Shamelessly stolen from Member Audit
from django.db import models

from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag

from .. import __title__

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access this app"),
            (
                "admin_access",
                "Can set tax rates for groups and add accountant characters",
            ),
            ("auditor_access", "Can view all registered characters data"),
        )



from django.apps import AppConfig

from . import __version__


class MiningTaxesConfig(AppConfig):
    name = "miningtaxes"
    label = "miningtaxes"
    verbose_name = f"Mining Taxes v{__version__}"



from django.core.management.base import BaseCommand

from ...tasks import update_daily


class Command(BaseCommand):
    help = "Runs daily update manually"

    def handle(self, *args, **options):
        update_daily()



# Generated by Django 4.0.7 on 2022-11-14 02:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0004_settings_interest_rate_alter_settings_phrase"),
    ]

    operations = [
        migrations.AddField(
            model_name="oreprices",
            name="raw_price",
            field=models.FloatField(default=0.0),
        ),
        migrations.AddField(
            model_name="oreprices",
            name="refined_price",
            field=models.FloatField(default=0.0),
        ),
        migrations.AddField(
            model_name="oreprices",
            name="taxed_price",
            field=models.FloatField(default=0.0),
        ),
    ]



from unittest.mock import patch

from allianceauth.eveonline.models import EveCharacter
from app_utils.testing import NoSocketsTestCase, add_new_token

from ...models import AdminCharacter
from ..testdata.esi_client_stub import esi_client_stub
from ..testdata.load_entities import load_entities
from ..testdata.load_eveuniverse import load_eveuniverse
from ..utils import create_character, create_miningtaxes_admincharacter

MODELS_PATH = "miningtaxes.models"


class TestAdminCharacter(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_eveuniverse()
        load_entities()

    def test_user_should_return_user_when_not_orphan(self):
        # given
        character_1001 = create_miningtaxes_admincharacter(1001)
        user = character_1001.eve_character.character_ownership.user
        # when/then
        self.assertEqual(character_1001.user, user)

    def test_user_should_be_None_when_orphan(self):
        # given
        character = create_character(EveCharacter.objects.get(character_id=1121))
        # when/then
        self.assertIsNone(character.user)

    def test_should_return_main_when_it_exists_1(self):
        # given
        character_1001 = create_miningtaxes_admincharacter(1001)
        user = character_1001.eve_character.character_ownership.user
        main_character = user.profile.main_character
        # when/then
        self.assertEqual(character_1001.main_character, main_character)

    @patch(MODELS_PATH + ".admin.esi")
    def test_corp_ledger(self, mock_esi):
        mock_esi.client = esi_client_stub
        # given
        character_1001 = create_miningtaxes_admincharacter(1001)
        user = character_1001.eve_character.character_ownership.user
        add_new_token(
            user, character_1001.eve_character, AdminCharacter.get_esi_scopes()
        )
        # when/then
        character_1001.update_all()
        ledger = character_1001.corp_ledger.all()
        self.assertEqual(len(ledger), 1)
        entry = ledger[0]
        self.assertEqual(entry.amount, 987654321)
        self.assertEqual(entry.reason, "")
        self.assertEqual(entry.taxed_id, 1001)

        observers = character_1001.mining_obs.all()
        self.assertEqual(len(observers), 1)
        obs = observers[0]
        self.assertEqual(obs.obs_id, 123456789)
        self.assertEqual(obs.name, "Amamake - Test Structure Alpha")
        self.assertEqual(obs.sys_name, "Amamake")

        mining_ledger = obs.mining_log.all()
        self.assertEqual(len(mining_ledger), 1)
        entry = mining_ledger[0]
        self.assertEqual(entry.miner_id, 1001)
        self.assertEqual(entry.quantity, 100)
        self.assertEqual(entry.eve_type_id, 45511)



from django.core.management.base import BaseCommand

from ...models import Stats
from ...tasks import update_all_prices


class Command(BaseCommand):
    help = "Preloads price data for ores"

    def handle(self, *args, **options):
        update_all_prices()
        s = Stats.load()
        s.calc_ore_prices_json()



# Generated by Django 4.0.7 on 2023-05-28 00:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0006_charactertaxcredits_credit_type"),
    ]

    operations = [
        migrations.AddField(
            model_name="character",
            name="life_credits",
            field=models.FloatField(default=0.0),
        ),
        migrations.AddField(
            model_name="character",
            name="life_taxes",
            field=models.FloatField(default=0.0),
        ),
    ]



from django.core.management.base import BaseCommand

from ...tasks import auto_add_chars


class Command(BaseCommand):
    help = "Runs auto adding of characters into the plugin"

    def handle(self, *args, **options):
        auto_add_chars()



# Shamelessly stolen from Member Audit
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from eveuniverse.models import EveType, EveTypeMaterial

from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag

from .. import __title__
from ..app_settings import (
    MININGTAXES_ALWAYS_TAX_REFINED,
    MININGTAXES_REFINED_RATE,
    MININGTAXES_UNKNOWN_TAX_RATE,
)

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def get_tax(eve_type):
    try:
        ore = OrePrices.objects.get(eve_type=eve_type)
        return ore.tax_rate / 100.0
    except OrePrices.DoesNotExist:
        pass
    return MININGTAXES_UNKNOWN_TAX_RATE


def ore_calc_prices(eve_type, q):
    try:
        ore = OrePrices.objects.get(eve_type=eve_type)
        return q * ore.raw_price, q * ore.refined_price, q * ore.taxed_price
    except OrePrices.DoesNotExist:
        pass
    quantity = q
    raw_price = quantity * get_price(eve_type)
    materials = EveTypeMaterial.objects.filter(
        eve_type_id=eve_type.id
    ).prefetch_related("eve_type")
    refined_price = 0.0
    for mat in materials:
        q = MININGTAXES_REFINED_RATE * (mat.quantity * quantity) / eve_type.portion_size
        refined_price += q * get_price(mat.material_eve_type)
    if refined_price == 0.0:
        refined_price = raw_price
    taxed_value = refined_price
    if raw_price > taxed_value:
        taxed_value = raw_price
    return raw_price, refined_price, taxed_value


def get_price(eve_type):
    ore = None
    try:
        ore = OrePrices.objects.get(eve_type=eve_type)
    except ObjectDoesNotExist:
        pass
    if ore is None:
        mp = None
        try:
            mp = eve_type.market_price
        except EveType.market_price.RelatedObjectDoesNotExist:
            pass
        if mp is None:
            return 0.0
        if eve_type.market_price.average_price is not None:
            return eve_type.market_price.average_price
        if eve_type.market_price.adjusted_price is not None:
            return eve_type.market_price.adjusted_price
        return 0.0
    return ore.buy


class OrePrices(models.Model):
    eve_type = models.OneToOneField(
        EveType,
        on_delete=models.deletion.CASCADE,
        unique=True,
    )
    buy = models.FloatField()
    sell = models.FloatField()
    raw_price = models.FloatField(default=0.0)
    refined_price = models.FloatField(default=0.0)
    taxed_price = models.FloatField(default=0.0)
    tax_rate = models.FloatField(default=10.0)
    updated = models.DateTimeField()

    def calc_prices(self):
        self.raw_price = self.buy
        materials = EveTypeMaterial.objects.filter(
            eve_type_id=self.eve_type.id
        ).prefetch_related("eve_type")
        if len(materials) == 0:  # try once if can't find materials in case of new ore
            EveTypeMaterial.objects.update_or_create_api(eve_type=self.eve_type)
            materials = EveTypeMaterial.objects.filter(
                eve_type_id=self.eve_type.id
            ).prefetch_related("eve_type")

        self.refined_price = 0.0
        for mat in materials:
            q = MININGTAXES_REFINED_RATE * mat.quantity / self.eve_type.portion_size
            self.refined_price += q * get_price(mat.material_eve_type)
        if self.refined_price == 0.0:
            self.refined_price = self.raw_price
        self.taxed_price = self.refined_price
        if not MININGTAXES_ALWAYS_TAX_REFINED and self.raw_price > self.taxed_price:
            self.taxed_price = self.raw_price
        self.save()
